#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "common.h"
#include "heuristics.h"

/* function heur_int_no_prev
   Calculate the heuristics value of a state and return it.
   Arguments: gamestate_t *state - the state
 */
int heur_int_no_prev(gamestate_t *state){
	h_state_t *h = heur_eval(state);
	int hval = h->val;
	free(h->board);
	free(h);
	return hval;
}

/* function heur_eval
   Calculate the heuristics state of a state and return it.
   Arguments: gamestate_t *state - the state
 */
h_state_t* heur_eval(gamestate_t *state){
	int i,j,/*ov,v,*/p,m = state->board->side;
	int pl,*b = state->board->b;
	pos_t *pos;
	if(state->last){
		pos = state->last;
		pl = -b[pos->y*m+pos->x];
	} else
		pl=-1;
	h_state_t *ret = malloc(sizeof(h_state_t));
	ret->val = 0;
	h_pos_t *hb;
	ret->board = calloc(sizeof(h_pos_t),m*m);
	hb = ret->board;
	for(j=0;j<m;j++){
		for(i=0;i<m;i++){
			p = j*m+i;
			//ov = ret->val;
			//No symbol here, no lines to search for
			if(!b[p])
				continue;
			//printf(".");
			//Look for lines, 
			if(!hb[p].vert)
				ret->val += update_vert(hb,b,i,j,m);
			if(!hb[p].horz)
				ret->val += update_horz(hb,b,i,j,m);
			if(!hb[p].diag)
				ret->val += update_diag(hb,b,i,j,m);
			if(!hb[p].diag2)
				ret->val += update_diag2(hb,b,i,j,m);
/*			if(ret->val - ov)
				printf("Hval changed by %d at (%d,%d)\n",ret->val -ov,
				i,j);*/
		}
	}
	ret->val *= pl;
	return ret;
}

/* function update_vert
   Scan the vertical line starting at the coordinates (x,y) in the board
   b, and update the corresponding heuristic board hb, return the
   difference to the heuristics value.
 */
int update_vert(h_pos_t *hb, int *b, int x, int y, int m){
	//printf("vert from (%d,%d): ",x,y);
	int player = b[y*m+x];
	int n = (y==0 || b[(y-1)*m+x]==-player) ? 0 : 1;
	int t,ret = 0;
	do{
		n+=4;
	}while(++y < m && b[y*m+x] == player);
	if(!(y==m || b[y*m+x]))
		n+=2;
	t = n >> 2;
	if((n & 3) == 0)
		n = 0;
	else if((n & 3)!=3)
		n = (n>>2) + 1;
	else	
		n = (n>>1) + 1;
	n++;
	n = player*n;
	while(t--){
		hb[(--y*m)+x].vert = n ;
		//printf("(%d,%d)",x,y);
		ret += n;
	}
	//printf("\n");
	return ret;
}

/* function update_horz
   Scan the horizontal line starting at the coordinates (x,y) in the board
   b, and update the corresponding heuristic board hb, return the
   difference to the heuristics value.
 */
int update_horz(h_pos_t *hb, int *b, int x, int y, int m){
	//printf("horz from (%d,%d): ",x,y);
	int player = b[y*m+x];
	int n = (x==0 || b[y*m+(x-1)]==-player) ? 0 : 1;
	int t,ret=0;
	do{
		n+=4;
	}while(++x < m && b[y*m+x] == player);
	if(!(x==m || b[y*m+x]))
		n+=2;
	t = n >> 2;
	if((n & 3) == 0)
		n = 0;
	else if((n & 3)!=3)
		n = (n>>2) + 1;
	else	
		n = (n>>1) + 1;
	n++;
	n = player*n;
	while(t--){
		hb[y*m+(--x)].horz = n;
		//printf("(%d,%d)",x,y);
		ret +=n;
	}
	//printf("\n");
	return ret;
}

/* function update_diag
   Scan the upper-left to lower-right diagonal line starting at the
   coordinates (x,y) in the board b, and update the corresponding heuristic
   board hb, return the difference to the heuristics value.
 */
int update_diag(h_pos_t *hb, int *b, int x, int y, int m){
	//printf("diag from (%d,%d): ",x,y);
	int player = b[y*m+x];
	int n = ((x==0 || y==0) || b[(y-1)*m+(x-1)]==-player) ? 0 : 1;
	int t,ret=0;
	do{
		n+=4;
	}while(++x < m && ++y < m && b[y*m+x] == player);
	if(x==m)
		++y;
	if(!(x==m || y==m || b[y*m+x]))
		n+=2;
	t = n >> 2;
	if((n & 3) == 0)
		n = 0;
	else if((n & 3)!=3)
		n = (n>>2) + 1;
	else	
		n = (n>>1) + 1;
	//n = (n>>1) + (n & 1);
	n++;
	n = player*n;
	while(t--){
		hb[(--y)*m+(--x)].diag = n;
		//printf("(%d,%d)",x,y);
		ret +=n;
	}
	//printf("\n");
	return ret;
}

/* function update_diag2
   Scan the upper-right to lower-left diagonal line starting at the
   coordinates (x,y) in the board b, and update the corresponding heuristic
   board hb, return the difference to the heuristics value.
 */
int update_diag2(h_pos_t *hb, int *b, int x, int y, int m){
	//printf("diag2 from (%d,%d): ",x,y);
	int player = b[y*m+x];
	int n = ((x==m || y==0) || b[(y-1)*m+(x+1)]==-player) ? 0 : 1;
	int t,ret=0;
	do{
		n+=4;
	}while(--x >= 0 && ++y < m && b[y*m+x] == player);
	if(x==-1)
		++y;
	if(!(x==0 || y==m || b[y*m+x]))
		n+=2;
	t = n >> 2;
	if((n & 3) == 0)
		n = 0;
	else if((n & 3)!=3)
		n = (n>>2) + 1;
	else	
		n = (n>>1) + 1;
	//n = (n>>1) + (n & 1);
	n++;
	n = player*n;
	while(t--){
		hb[(--y)*m+(++x)].diag = n;
		//printf("(%d,%d)",x,y);
		ret +=n;
	}
	//printf("\n");
	return ret;
}

int update(h_pos_t *hb, int *b, int x, int y, int m, int xc, int yc){
	//printf("diag2 from (%d,%d): ",x,y);
	int player = b[y*m+x];
	int n = ((x==m || y==0) || b[(y-yc)*m+(x+xc)]==-player) ? 0 : 1;
	int t,ret=0;
	do{
		n+=4;
		x+=xc;
		y+=yc;
	}while(x >= 0 && y < m && b[y*m+x] == player);
	if(x==-1)
		++y;
	if(!(x==0 || y==m || b[y*m+x]))
		n+=2;
	t = n >> 2;
	if((n & 3) == 0)
		n = 0;
	else if((n & 3)!=3)
		n = (n>>2) + 1;
	else	
		n = (n>>1) + 1;
	//n = (n>>1) + (n & 1);
	n++;
	n = player*n;
	while(t--){
		hb[(--y)*m+(++x)].diag = n;
		//printf("(%d,%d)",x,y);
		ret +=n;
	}
	//printf("\n");
	return ret;
}

/*
void recalc(int *v,int* b,int m, h_pos_t* hb, pos_t *pos,int xc, int yc){

}

h_state_t* heur(h_state_t *prev, gamestate_t *curr){
	pos_t *pos=curr->last;
	h_state_t *next = malloc(sizeof(h_state_t));
	int *b = curr->board->b, m = curr->board->side;
	int pl=b[p],val = prev->val;
	h_pos_t *hb;
	next->board = malloc(sizeof(h_pos_t)*m*m);
	memcpy(next->board,prev->board,sizeof(h_pos_t)*m*m);
	hb = next->board;
	recalc(&val,b,m,hb,pos,1,0);
	recalc(&val,b,m,hb,pos,0,1);
	recalc(&val,b,m,hb,pos,1,1);
	recalc(&val,b,m,hb,pos,-1,1);
	next->val = val;
	return next;
}*/


