#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>

#include "sighant.h"
#include "common.h"
#include "main.h"
#include "heuristics.h"

extern int continuesearch;

/**
 * Bra Struct (tm)
 */
typedef struct listnode {
  struct listnode *next;
  gamestate_t *state;
} listnode_t;


/**
 * usage: newlist = listadd(oldlist, state);
 */
listnode_t *listadd(listnode_t *list, gamestate_t *state) {
  listnode_t *newnode = malloc(sizeof(listnode_t));
  newnode->state = state;
  newnode->next = list;
  return newnode;
}

/**
 * Usage: list and a pointer to a state pointer. Returns the
 * tail of the list and sets the poitner
 */
listnode_t *listremove(listnode_t *list, gamestate_t **state){
  listnode_t *tail=list->next;
  *state = list->state;
  free(list);
  return tail;
}

/**
 * TODO: also free() node contents.
 */
void listfree(listnode_t *list) {
  listnode_t *temp = list;
  while(list) {
    list = list->next;
    free(temp->state->board);
    free(temp->state->last);
    free(temp->state);
    free(temp);
    temp = list;
  }
}

/**
 * Gives size of list.
 */
int listnum(listnode_t *list) {
  int i = 0;
  while(list != NULL) {
    i++;
    list = list->next;
  }
  return i;
}

/**
 * Should give a linked list of successor states.
 */
listnode_t *expand(gamestate_t *state) {
  board_t *board = state->board;
  pos_t *last = state->last;
  int side = state->board->side;
  int mark;

  if (last == NULL) {
    mark = 1;
  } else {
    mark = board->b[last->y*side + last->x];
  }
  mark = -1 * mark; //dvs nextmark
  listnode_t *list = NULL;
  gamestate_t *newstate;

  int i, j;
  for (i = 0; i < side; i++) {
    for (j = 0; j < side; j++) {
      if (board->b[i*side+j])
        continue;
      newstate = malloc(sizeof(gamestate_t));
      newstate->board = malloc(sizeof(int)*(1+ side*side));
      newstate->last = malloc(sizeof(pos_t));
      newstate->last->x = j;
      newstate->last->y = i;
      memcpy(newstate->board, state->board, sizeof(int)*(1+ side*side));
      newstate->board->b[i*side+j] = mark;
      list = listadd(list, newstate);
    }
  }
  return list;
}

int max(int a, int b) {
  if (a > b) return a; else return b;
}

int alphabeta(gamestate_t *state, int depth, int alpha, int beta, int n, pos_t *bestp) {
  int acandidate;
  if (!continuesearch) {
    return alpha;
  }
  /*If gameover, I've lost.*/
  if (gameover(state, n)) {
    return -(10000+depth);
  }
  /*If reached maximum depth, return heuristic value of node*/
  if (depth == 0) {
    return heur_int_no_prev(state);
  }

  listnode_t *list = expand(state);


  gamestate_t *currstate;

  while (list != NULL) {
    list = listremove(list, &currstate);
    
    acandidate = -alphabeta(currstate, depth-1, -beta, -alpha, n, NULL);
    if (acandidate > alpha) {
      alpha = acandidate;
      if (bestp != NULL && continuesearch) {
        bestp->x = currstate->last->x;
        bestp->y = currstate->last->y;
      }
    }
    free(currstate->board);
    free(currstate->last);
    free(currstate);
    if (beta <= alpha) {
      listfree(list);
      break;
    }
  }
  return alpha;
}
