#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#include "common.h"
#include "main.h"
#include "random.h"
#include "heuristics.h"
#include "search.h"
#include "aiplayer.h"

static board_t *main_board;
static char alphabet[] = {
	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B',
	'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'};

int main(int argc, char* argv[]) {
	player_t *ui,*p[2];
	gamestate_t *endstate;
	int n,m,d,begin;

	if(argc != 4 && argc != 5){
		printusage(argv[0]);
		exit(-1);
	}
	if(sscanf(argv[1],"%d",&n)!=1){
		printf("Invalid n, must be a number");
		printusage(argv[0]);
		exit(-1);
	}
	if(sscanf(argv[2],"%d",&m)!=1){
		printf("Invalid m, must be a number");
		printusage(argv[0]);
		exit(-1);
	}
	if(m>52){
		printf("Invalid m, must be 52 or less");
		printusage(argv[0]);
		exit(-1);
	}
	if(sscanf(argv[3],"%d",&d)!=1){
		printf("Invalid d, must be a number");
		printusage(argv[0]);
		exit(-1);
	}
	begin = argc-4;

	p[1] = ai_get_player(m, n, d);
	ui = malloc(sizeof(player_t));
	ui->next_move = get_player_ply;
	p[0] = ui;
	endstate = game(n,m,p[begin],p[1-begin]);
//	ui_free_quit(endstate);
	ai_free_quit(endstate);
//	random_free_quit(endstate);
	cleanup(endstate);
	free(ui); //denna r ny
	return 0;
}

gamestate_t* game(int n, int m, player_t *p1, player_t *p2){
	player_t *t;
	pos_t *move=NULL;
	gamestate_t *state = malloc(sizeof(gamestate_t));
	main_board = calloc(sizeof(int),(m*m+1));
	main_board->side = m;
	state->board = malloc(sizeof(int)*(m*m+1));
	memcpy(state->board, main_board, sizeof(int)*(m*m+1));
	state->last =NULL;
	do{

		//printf("Hval: %d\n",heur_int_no_prev(state));
		move = p1->next_move(state);
		if(update_board(main_board,move)){
			printf("fejl\n");
			break;
		}
		free(state->board);
		free(state->last);
		state->board = malloc(sizeof(int)*(m*m+1));
		memcpy(state->board, main_board, sizeof(int)*(m*m+1));
		state->last = move;
		//printstate(state);
		t = p1;
		p1 = p2;
		p2 = t;
	}while(!gameover(state, n));
	free(state->board);
	state->board = main_board;
	state->last = move;
	free(move);
	return state;
}

int update_board(board_t* board, pos_t *move){
	static int turn = 1;
	unsigned int m =board->side;
	int *p = board->b + move->y*m + move->x;
	if((move->y > m) || (move->x > m))
		return -1;
	if(*p)
		return -1;
	*p = turn;
	turn = -turn;
	return 0;
}

int checkline(gamestate_t *state,int xc, int yc){
	pos_t p = *(state->last);
	int m = state->board->side;
	int s = state->board->b[p.y*m+p.x];
	int n = 0;
	int *b = state->board->b;
	do{
		if(b[p.y*m+p.x] == s)
			n++;
		else
			break;
		p.x += xc;
		p.y += yc;
	}while(p.x < m && p.x >=0 && p.y < m && p.y >= 0);
	p = *(state->last);
	do{
		if(b[p.y*m+p.x] == s)
			n++;
		else
			break;
		p.x -= xc;
		p.y -= yc;
	}while(p.x < m && p.x >=0 && p.y < m && p.y >= 0);
	return n;	
}


int gameover(gamestate_t *state, int n){
	
	int i,j,*b,m;

	if(!state->last)
		return 0;
	for(i=-1;i<=1;i++)
		if(checkline(state,1,i)>n)
			return 1;
	if(checkline(state,0,1)>n)
		return 1;
	b = state->board->b;
	m = state->board->side;
	for(j=0;j<m;j++)
		for(i=0;i<m;i++)
			if(!b[j*m+i])
				return 0;
	return 1;
}

void cleanup(gamestate_t *end){
	printstate(end);
	free(end->board);
	free(end);
}

void printstate(gamestate_t *end){
	int i,j,s = end->board->side;
	printf("----------------\n");
	for(j=0;j<s;j++)
		printf("%c",alphabet[j]);
	printf("\n");
	for(j=0;j<s;j++){
		for(i=0;i<s;i++)
			if(end->board->b[j*s+i]>0)
				printf("X");
			else if(end->board->b[j*s+i]<0)
				printf("O");
			else
				printf(" ");
		printf(" %c\n",alphabet[j]);
	}
	printf("----------------\n");
}

pos_t* get_player_ply(gamestate_t *state){
	unsigned int x, y, s = state->board->side;
	char c,str[100];
	pos_t *p;
	printstate(state);
	do{
		do{
			printf("X: ");
			fgets(str,100,stdin);
			sscanf(str,"%c",&c);
			fflush(stdin);
			for(x = 0; x <= s && alphabet[x] != c;x++);
		}while(x >= s);
		do{
			printf("Y: ");
			fgets(str,100,stdin);
			sscanf(str,"%c",&c);
			fflush(stdin);
			for(y = 0; y <= s && alphabet[y] != c;y++);
		}while(y >= s);
	}while(state->board->b[y*state->board->side+x]);
	p = malloc(sizeof(pos_t));
	p->x = x;
	p->y = y;
	return p;
}


void printusage(char* s){
	printf("Usage: %s n m d [foo]\nn: number of tic or tac in a row\n"
			"m: size of game board (52 or less)\n"
			"d: depth of AI search tree\n"
			"if there is a fourth argument, the AI begins play\n",s);
}



