import java.io.*;
import java.util.ArrayList;
import java.util.List;


public class Bayes {

	public Bayes(File f) {
		BufferedReader r = null;
		try {
			r = new BufferedReader(new FileReader(f));
		} catch (FileNotFoundException e) {
			System.err.println("Could not find file!");
			System.exit(1);
		}
		String line;
		List<String> linelist = new ArrayList<String>();
		try {
			while ((line = r.readLine()) != null) {
				line = line.replaceAll(" ", ""); // Remove spaces.
				if (!line.equals("") && !line.startsWith("#")) {
					linelist.add(line);
				}
			}
		} catch (IOException e) {
			System.err.println("Error when reading file!");
			System.exit(1);
		}
		
		Network network = new Network(linelist);
		for (String s : linelist) {
			if (s.toUpperCase().startsWith("Q")) {
				String s2 = s.substring(s.indexOf('(')+1, s.indexOf(')'));
				String[] sides = s2.split("\\|");
				String[] lefts = sides[0].split(",");
				String[] rights = new String[0];
				if (sides.length == 2) {
					rights = sides[1].split(",");
				}
				List<Identifier> asks = new ArrayList<Identifier>();
				ArrayList<Identifier> conditionals = new ArrayList<Identifier>();
				for (String id : lefts) {
					asks.add(Network.stringToIdentifier(id));
				}
				for (String id2 : rights) {
					conditionals.add(Network.stringToIdentifier(id2));
				}
				double v = network.Prob(asks, conditionals);
				System.out.print(s+" = "+v+"\n");
			}
		}
	}

	public static void main(String[] args) {
		if (args.length != 1) {
			System.out.println("Usage: java -jar Bayes.jar <network file>");
			System.exit(1);
		}
		File f = new File(args[0]);
		new Bayes(f);
	}
}
