package ai;

import java.io.*;
import java.util.*;


/**
 * Represents a feed-forward neural net wherein every node in a layer
 * has every node in the next layer as its neighbour.
 * 
 * @author Eric Jnsson (ericj@cs.umu.se)
 */
public class NeuralNet implements Serializable {
	
    private static final long serialVersionUID = 1L;
    private final double[][] w;
	private int in;
	private int out;
	private double a;
	private Random rand;
	
	public NeuralNet(double a, int in, int out) {
		this.in = in;
		this.out = out;
		this.a = a;
		w = new double[in][out];
		rand = new Random();
		for (int i = 0; i < in; i++)
			for (int j = 0; j < out; j++)
				w[i][j] = 1;
	}

	private double[] err(double[] guess, int[] cr) {
		double[] res = new double[out];
		for (int i = 0; i < out; i++) {
			res[i] = cr[i] - guess[i];
		}
		return res;
	}
	
	private double update(int[] input, double[] guess, double[] error) {
		double change = 0;
		for (int i = 0; i < out; i++) {
			for (int j = 0; j < in; j++) {
				w[j][i] += a * error[i] * gp(guess[i]) * input[j];
				change += Math.abs(a * error[i] * gp(guess[i]) * input[j]);
			}
		}
		return change;
	}
	
	public void train(int[][] ex, int[][] cr) {
		
		int epoch = 100;
		boolean ok = false;
		while (!ok) {
			double change = 0;
			for (int e = 0; e < epoch; e++) {
				int r = rand.nextInt(ex.length);
				double[] guess = calc(ex[r]);
				double[] error = err(guess, cr[r]);
				change += update(ex[r], guess, error);
			}
//			System.out.println("Change: "+change);
			if (change < epoch * 10)
				ok = true;
		}

	}
	
	
	public double[] calc(int[] ex) {
		double[] res = new double[out];
		for (int i = 0; i < out; i++) {
			double sum = 0;
			for (int j = 0; j < in; j++) {
				sum += w[j][i] * ex[j];
			}
			res[i] = g(sum);
		}
		return res;
	}
	
	private double g(double x) {
		return 1/(1 + Math.exp(-x));
	}

	private double gp(double x) {
		return g(1 - g(x));
	}
}
