package lab3;

import java.io.*;
import java.util.*;
import ai.*;
import pic.*;

/**
 * Entry point of lab3.
 * @author eric
 *
 */
public class Faces {
	
    private static void save(Serializable obj) throws IOException {
        ObjectOutputStream os = new ObjectOutputStream(
                new FileOutputStream("matrix.dat")
        );
        os.writeObject(obj);
        os.close();
    }
    
    private static Object load() throws IOException, ClassNotFoundException {
        ObjectInputStream os = new ObjectInputStream(
                new FileInputStream("matrix.dat")
                );
        Object obj = os.readObject();
        os.close();
        return obj;
    }
    
	private static int translate(double[] res) {
		double max = res[0];
		int l = 0;
		for (int i = 1; i < res.length; i++) {
			if (res[i] > max)
				l = i;
		}
		return l + 1;
	}
	
	private static void printUsage() {
	    System.err.println("Faces, AI lab3");
	    System.err.println("3 arguments needed: [train input.dat correct.dat|guess input.dat]");
	    System.err.println("- With the 'train' switch, trains itself on the training data");
	    System.err.println("- With the 'guess' switch, outputs face guesses to stdout.");
	    System.err.println("Example: java -jar Faces.jar train training.dat correct.dat");
	    System.err.println("Example: java -jar Faces.jar guess test.dat");
	}
	
	public static void main(String[] args) {
	    
	    int[][] img = null;
        int[][] cr = null;
        int outsize = 4;
        NeuralNet net = null;
	    
	    if (args.length > 1) {
	        if (args[0].equals("train") && args.length == 3) {
	            try {
	                img = FaceFactory.buildFaceMatrix(args[1]);
	                cr = FaceFactory.buildCorrectMatrix(args[2], img.length, outsize);
	                net = new NeuralNet(0.2, img.length, outsize);
	                net.train(img, cr);
	                save(net);
	            } catch (IOException e) {
	                System.err.println("Failed reading files.");
	                System.exit(1);
	            }
	            System.out.println("Perceptron trained. Bring it on!");
	        } else if (args[0].equals("guess") && args.length == 2) {
	            List<Face> faces = null;
	            try {
                    img = FaceFactory.buildFaceMatrix(args[1]);
                    faces = FaceFactory.buildFaceList(args[1]);
                    net = (NeuralNet)load();
                } catch (Exception e) {
                    System.err.println("Failed reading file.");
                    System.exit(1);
                }
                for (int i = 0; i < img.length; i++) {
                    int guess = translate(net.calc(img[i]));
                    System.out.println(faces.get(i).getTag() + " " + guess);
                }
	        } else {
	            printUsage();
	            System.exit(1);
	        }
	    } else {
	        printUsage();
            System.exit(1);
	    }
	    
		
		
	}
}
