package pic;

/**
 * Face, a wrapper around an int-array with some convenience methods.
 * @author eric
 *
 */
public class Face {

    private final int[] data;
    private final String tag;
    private final int width;
    private final int height;
    private FaceType type;
    
    /**
     * Creates a face
     * @param t The face tag.
     * @param h Height in pixels
     * @param w Width in pixels
     */
    public Face(String t, int h, int w) {
    	tag = t;
        data = new int[h * w];
        width = w;
        height = h;
    }
    
    /**
     * Sets the face type
     * @param t The face type.
     */
    public void setType(FaceType t) {
    	type = t;
    }
    
    /**
     * @return The type of this Face.
     */
    public FaceType getType() {
    	return type;
    }
    
    /**
     * Sets a single value in the bitmap.
     * @param pos Position, 0-indexed.
     * @param val The value to set.
     */
    public void setValue(int pos, int val) {
    	data[pos] = val;
    }
    
    /**
     * Sets a single value in the bitmap.
     * @param x X-coordinate
     * @param y Y-coordinate
     * @param val Value to set.
     */
    public void setValue(int x, int y, int val) {
    	setValue(y * width + x, val);
    }
    
    /**
     * @return The image data.
     */
    public int[] getData() {
        return data;
    }
    
    /**
     * @return The image tag.
     */
    public String getTag() {
    	return tag;
    }
    
    /**
     * @return The height of this picture.
     */
    public int getHeight() {
    	return height;
    }
    
    /**
     * @return The width of this picture.
     */
    public int getWidth() {
    	return width;
    }
}
