package pic;

import java.io.*;
import java.util.*;

/**
 * Factory class for creating faces
 * @author eric
 *
 */
public class FaceFactory {

	/**
	 * Creates an integer matrix of image data from two text files.
	 * @param imgpath Path to image data file
	 * @return Integer matrix of image data.
	 * @throws IOException
	 */
	public static int[][] buildFaceMatrix(String imgpath) 
	throws IOException {
		List<Face> faces = buildFaceList(imgpath);
		int[][] res = new int[faces.size()][];
		for (int i = 0; i < faces.size(); i++) {
			res[i] = faces.get(i).getData();
		}
		return res;
	}
	
	/**
	 * Builds an array of integers from a file of correct faces.
	 * @param correctpath Path to file
	 * @param size Number of rows in file
	 * @return Integer array of correct faces
	 * @throws IOException
	 */
	public static int[][] buildCorrectMatrix(String path, int size, int len)
	throws IOException {
		BufferedReader input = null;
		int[][] result = new int[size][len];
		try {
			input = new BufferedReader(new FileReader(new File(path)));
			for (int i = 0; i < size; i++) {
				int n = Integer.parseInt(input.readLine().split(" ")[1]);
				for (int j = 0; j < len; j++) {
					if (j == n-1)
						result[i][j] = 1;
					else
						result[i][j] = 0;
				}
			}
		} finally {
			if (input != null)
				input.close();
		}
		return result;
	}
	
	/**
	 * Creates a list of faces from two text files.
	 * @param imgpath Path to image data file
	 * @return A list of faces.
	 * @throws IOException
	 */
	public static List<Face> buildFaceList(String imgpath)
	throws IOException {
		BufferedReader input = null;
		List<Face> result = null;
		try {
			input = new BufferedReader(new FileReader(new File(imgpath)));
			input.readLine();
			int reqbits = Integer.parseInt(input.readLine());
			if (reqbits > 32) {
				throw new IOException("Unsupported file format");
			}
			int numpix = Integer.parseInt(input.readLine());
			result = new ArrayList<Face>(numpix);
			String[] tmp = input.readLine().split(" ");
			int width = Integer.parseInt(tmp[0]);
			int height = Integer.parseInt(tmp[1]);
			for(int count = 0; count < numpix; count++) {
				input.readLine();
				String tag = input.readLine();
				Face f = new Face(tag, height, width);
				for (int row = 0; row < height; row++) {
					String[] line = input.readLine().split(" ");
					for (int col = 0; col < width; col++) {
						f.setValue(col, row, Integer.parseInt(line[col]));
					}
				}
				result.add(f);
			}

		} finally {
			if (input != null) {
				input.close();
			}
		}
		return result;
	}
}
